import pytest
from sphinx.testing.path import path


@pytest.fixture(scope="module")
def rootdir():
    return path(__file__).parent.abspath() / "roots" / "builder"


@pytest.mark.sphinx("rediraffewritediff", testroot="renamed_write_file_not_redirected")
def test_builder_renamed_file_write_not_redirected(app_init_repo):
    app_init_repo.build()
    valid_string = '"another.rst" "another2.rst"'
    with open(path(app_init_repo.srcdir).joinpath("redirects.txt"), "r") as file:
        assert valid_string in file.readline()


@pytest.mark.sphinx("rediraffewritediff", testroot="renamed_write_file_perc_low_fail")
def test_builder_renamed_file_write_perc_low_fail(app_init_repo):
    app_init_repo.build()
    valid_string = '"another.rst" "another2.rst"'
    with open(path(app_init_repo.srcdir).joinpath("redirects.txt"), "r") as file:
        assert valid_string not in file.readline()


@pytest.mark.sphinx("rediraffewritediff", testroot="renamed_write_file_perc_low_pass")
def test_builder_renamed_file_write_perc_low_pass(app_init_repo):
    app_init_repo.build()
    valid_string = '"another.rst" "another2.rst"'
    with open(path(app_init_repo.srcdir).joinpath("redirects.txt"), "r") as file:
        assert valid_string in file.readline()



import pytest
from sphinx.testing.path import path


@pytest.fixture(scope="module")
def rootdir():
    return path(__file__).parent.abspath() / "roots" / "builder"


@pytest.mark.sphinx("rediraffecheckdiff", testroot="file_changed")
def test_file_changed(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 0


@pytest.mark.sphinx("rediraffecheckdiff", testroot="deleted_file_redirected")
def test_builder_deleted_file_redirected(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 0


@pytest.mark.sphinx("rediraffecheckdiff", testroot="deleted_file_not_redirected")
def test_builder_deleted_file_not_redirected(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 1


@pytest.mark.sphinx("rediraffecheckdiff", testroot="renamed_file_redirected")
def test_builder_renamed_file_redirected(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 0


@pytest.mark.sphinx("rediraffecheckdiff", testroot="renamed_file_not_redirected")
def test_builder_renamed_file_not_redirected(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 1


@pytest.mark.sphinx("rediraffecheckdiff", testroot="link_redirected_to_chain")
def test_builder_link_redirected_to_chain(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 0


@pytest.mark.sphinx("rediraffecheckdiff", testroot="link_redirected_twice")
def test_builder_link_redirected_twice(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 1


@pytest.mark.sphinx("rediraffecheckdiff", testroot="bad_rediraffe_file")
def test_builder_bad_rediraffe_file(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 1


@pytest.mark.sphinx("rediraffecheckdiff", testroot="nested")
def test_builder_nested(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 0


@pytest.mark.sphinx("rediraffecheckdiff", testroot="backslashes")
def test_builder_backslashes(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 0


@pytest.mark.sphinx("rediraffecheckdiff", testroot="mixed_slashes")
def test_builder_mixed_slashes(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 0


@pytest.mark.sphinx("rediraffecheckdiff", testroot="deleted_file_redirected_commit")
def test_builder_deleted_file_redirected_commit(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 0


@pytest.mark.sphinx("rediraffecheckdiff", testroot="deleted_file_not_redirected_commit")
def test_builder_deleted_file_not_redirected_commit(app_init_repo):
    app_init_repo.build()
    assert app_init_repo.statuscode == 1



from typing import Dict, List, Union
from sphinx.testing.path import path
import pytest
import subprocess
from pathlib import Path
import os
import stat
import shutil
from contextlib import suppress
from seleniumbase import config as sb_config

pytest_plugins = "sphinx.testing.fixtures"


def del_rw(action, name, exc):
    os.chmod(name, stat.S_IWRITE)
    os.remove(name)


def delete(path: Union[str, Path]):
    path = Path(path).resolve()
    if path.exists():
        if path.is_dir():
            shutil.rmtree(path, onerror=del_rw)
        else:
            try:
                path.unlink()
            except Exception:
                del_rw(None, path, None)


# @pytest.fixture(scope="session")
# def rootdir():
#     return path(__file__).parent.abspath() / "roots"


@pytest.fixture()
def content(app):
    app.build()
    yield app


@pytest.fixture()
def outdir(app):
    return app.outdir


@pytest.fixture()
def app_init_repo(make_app, app_params):
    """
    Initializes a git repo before returning the app.

    The files in the specified test root are converted into a git repo.
    For example, in a test root with folders: HEAD, HEAD~1, HEAD~2,
    the files in each folder are treated as a snapshot of a git repo at
    their respective revs.
    A git repo is created and in order:
    1. HEAD~2's files are moved to the parent and are commited
    2. HEAD~1's files are moved to the parent and are commited
    3. HEAD's files are moved to the parent and are commited


    Returns:
    app: Sphinx app; same as the `app` fixture
    """

    args, kwargs = app_params

    srcdir = None
    if "srcdir" in kwargs:
        srcdir = kwargs["srcdir"]
    elif "buildername" in kwargs:
        srcdir = args[1]
    else:
        srcdir = args[0]

    print(srcdir)
    src_path = Path(srcdir)

    def git(cmd: str):
        with suppress(subprocess.CalledProcessError):
            subprocess.check_output(
                "git"
                ' -c user.name="sphinxext-linkcheckdiff test runner"'
                ' -c user.email="NONE"'
                f" -C {src_path}"
                f" {cmd}",
                shell=True,
            )

    git("init")

    src_item_paths = list(src_path.glob("*"))

    commits = []

    for item_path in src_item_paths:

        if not item_path.name.startswith("HEAD"):
            continue

        commit_num = None

        sq_idx = item_path.name.find("~")
        if sq_idx != -1:
            commit_num = int(item_path.name[sq_idx + 1 :])
        else:
            commit_num = 0

        commits.append((item_path, commit_num))

    commits.sort(key=lambda tup: tup[1], reverse=True)

    for commit in commits:
        files = list(src_path.glob("*"))

        for file in files:
            if file not in src_item_paths:
                delete(file)

        for file in (src_path / commit[0]).glob("*"):
            new_path = file.parent.parent / file.name
            file.rename(new_path)

        git("add .")
        for path in src_item_paths:
            git(f"rm --cached -r {path}")

        git(f'commit -m "Apply {commit[0]}"')

        delete(src_path / commit[0])

    return make_app(*args, **kwargs)


def pytest_configure(config):
    config.addinivalue_line("markers", "sphinx")


def rel2url(outdir, path):
    return "file://" + str((Path(outdir) / path).resolve())


@pytest.fixture(scope="session")
def _sb(request):
    """ Same as the sb fixture but with a session scope"""
    from seleniumbase import BaseCase

    class BaseClass(BaseCase):
        def setUp(self):
            super(BaseClass, self).setUp()

        def tearDown(self):
            self.save_teardown_screenshot()
            super(BaseClass, self).tearDown()

        def base_method(self):
            pass

    sb = BaseClass("base_method")
    sb.setUp()
    sb._needs_tearDown = True
    sb_config._sb_node[request.node.nodeid] = sb
    yield sb
    if sb._needs_tearDown:
        sb.tearDown()
        sb._needs_tearDown = False


@pytest.fixture()
def ensure_redirect(outdir, _sb):
    # outdir = app.outdir
    def _ensure_redirect(before: str, expected_after: str):
        before_url = rel2url(outdir, before)
        _sb.open(before_url)
        expected_after_path = Path(rel2url(outdir, expected_after))
        actual_after_path = Path(_sb.get_current_url())
        assert actual_after_path == expected_after_path

    return _ensure_redirect



extensions = ["sphinxext.rediraffe"]

master_doc = "index"
exclude_patterns = ["_build"]

html_theme = "basic"

rediraffe_redirects = "redirects.txt"

import subprocess
from pathlib import Path

rediraffe_branch = subprocess.check_output(
    f"git -C {Path(__file__).parent} rev-parse HEAD~1", shell=True
).decode("utf-8")



import json
from os import rename
import re
import subprocess
from os.path import relpath
from pathlib import Path, PurePath, PureWindowsPath, PurePosixPath
from typing import Any, Dict, List, Union

from jinja2 import Environment, FileSystemLoader, Template
from sphinx.application import Sphinx
from sphinx.builders import Builder
from sphinx.builders.dirhtml import DirectoryHTMLBuilder
from sphinx.builders.html import StandaloneHTMLBuilder
from sphinx.builders.linkcheck import CheckExternalLinksBuilder
from sphinx.errors import ExtensionError
from sphinx.util import logging
from sphinx.util.console import green, red, yellow  # pylint: disable=no-name-in-module

logger = logging.getLogger(__name__)

DEFAULT_REDIRAFFE_TEMPLATE = Template(
    """
<html>
    <head>
        <noscript>
            <meta http-equiv="refresh" content="0; url={{rel_url}}"/>
        </noscript>
    </head>
    <body>
        <script>
            window.location.href = '{{rel_url}}' + (window.location.search || '') + (window.location.hash || '');
        </script>
        <p>You should have been redirected.</p>
        <a href="{{rel_url}}">If not, click here to continue.</a>
    </body>
</html>

"""
)
REDIRECT_JSON_NAME = "_rediraffe_redirected.json"
RE_OBJ = re.compile(r"(?:(\"|')(.*?)\1|(\S+))\s+(?:(\"|')(.*?)\4|(\S+))")

READTHEDOCS_BUILDERS = ["readthedocs", "readthedocsdirhtml"]


def create_graph(path: Path) -> Dict[str, str]:
    """
    Convert a file containing a whitespace delimited edge list (key value pairs) to a dict. Throws error on duplicate keys.
    """
    graph_edges = {}
    broken = False
    with open(path, "r") as file:
        for line_num, line in enumerate(file, start=1):
            line = line.strip()
            if len(line) == 0 or line.startswith("#"):
                continue
            match = RE_OBJ.fullmatch(line)

            if match == None:
                logger.error(
                    red(f"rediraffe: line {line_num} of the redirects is invalid!")
                )
                broken = True
                continue

            edge_from = match.group(2) or match.group(3)
            edge_to = match.group(5) or match.group(6)
            if edge_from in graph_edges:
                # Duplicate vertices not allowed / Vertices can only have 1 outgoing edge
                logger.error(
                    red(
                        f"rediraffe: {edge_from} is redirected multiple times in the rediraffe_redirects file!"
                    )
                )
                broken = True
            graph_edges[edge_from] = edge_to
    if broken:
        err_msg = f"rediraffe: Error(s) in parsing the redirects file."
        logger.error(err_msg)
        raise ExtensionError(err_msg)
    return graph_edges


def create_simple_redirects(graph_edges: dict) -> dict:
    """
    Ensures that a graph is a acyclic and reconnects every vertex to its leaf vertex.
    """
    redirects = {}
    broken_vertices = set()
    for vertex in graph_edges:
        if vertex in broken_vertices:
            continue

        visited = []
        while vertex in graph_edges:
            if vertex in visited:
                # Ensure graph is a DAG
                logger.error(
                    red(
                        "rediraffe: A circular redirect exists. Links involved: "
                        + " -> ".join(visited + [vertex])
                    )
                )
                broken_vertices.update(visited)
                break
            visited.append(vertex)
            vertex = graph_edges[vertex]

        # vertex is now a leaf
        for visited_vertex in visited:
            redirects[visited_vertex] = vertex

    if broken_vertices:
        err_msg = (
            f"rediraffe: At least 1 circular redirect detected. All involved links: "
            + ", ".join(broken_vertices)
        )
        logger.error(err_msg)
        raise ExtensionError(err_msg)

    return redirects


def remove_suffix(docname: str, suffixes: List[str]) -> str:
    """Remove any known suffixes for a file path."""
    for suffix in suffixes:
        if docname.endswith(suffix):
            return docname[: -len(suffix)]
    return docname


def build_redirects(app: Sphinx, exception: Union[Exception, None]) -> None:
    """
    Build amd write redirects
    """
    redirect_json_file = Path(app.outdir) / REDIRECT_JSON_NAME
    if redirect_json_file.exists():
        redirect_record = json.loads(redirect_json_file.read_text("utf8"))
    else:
        redirect_record = {}

    if exception != None:
        return

    if isinstance(app.builder, CheckExternalLinksBuilder):
        logger.info("rediraffe: Redirect generation skipped for linkcheck builders.")
        return

    if (
        type(app.builder) not in (StandaloneHTMLBuilder, DirectoryHTMLBuilder)
        and app.builder.name not in READTHEDOCS_BUILDERS
    ):
        logger.info(
            "rediraffe: Redirect generation skipped for unsupported builders. Supported builders: html, dirhtml, readthedocs, readthedocsdirhtml."
        )
        return

    rediraffe_template = app.config.rediraffe_template
    if isinstance(rediraffe_template, str):
        # path
        template_path = Path(app.srcdir) / rediraffe_template
        if template_path.exists():
            file_loader = FileSystemLoader(template_path.parent)
            env = Environment(loader=file_loader)
            rediraffe_template = env.get_template(template_path.name)
        else:
            logger.warning(
                "rediraffe: rediraffe_template does not exist. The default will be used."
            )
            rediraffe_template = DEFAULT_REDIRAFFE_TEMPLATE
    else:
        rediraffe_template = DEFAULT_REDIRAFFE_TEMPLATE

    graph_edges = {}

    rediraffe_redirects = app.config.rediraffe_redirects
    if isinstance(rediraffe_redirects, dict):
        # dict in conf.py
        graph_edges = rediraffe_redirects
    elif isinstance(rediraffe_redirects, str):
        # filename
        path = Path(app.srcdir) / rediraffe_redirects
        if not path.is_file():
            logger.error(
                red(
                    "rediraffe: rediraffe_redirects file does not exist. Redirects will not be generated."
                )
            )
            app.statuscode = 1
            return

        try:
            graph_edges = create_graph(path)
        except ExtensionError as e:
            app.statuscode = 1
            raise e
    else:
        logger.warning(
            "rediraffe: rediraffe was not given redirects to process. Redirects will not be generated."
        )
        return

    try:
        redirects = create_simple_redirects(graph_edges)
    except ExtensionError as e:
        app.statuscode = 1
        raise e

    logger.info("Writing redirects...")

    # write redirects
    for src_redirect_from, src_redirect_to in redirects.items():
        # Normalize path - src_redirect_.* is relative so drive letters aren't an issue.
        src_redirect_from = Path(PureWindowsPath(src_redirect_from))
        src_redirect_to = Path(PureWindowsPath(src_redirect_to))

        # remove extensions
        redirect_from_name = remove_suffix(
            src_redirect_from.name, app.config.source_suffix
        )
        redirect_to_name = remove_suffix(src_redirect_to.name, app.config.source_suffix)

        redirect_from = src_redirect_from.parent / f"{redirect_from_name}.html"
        redirect_to = src_redirect_to.parent / f"{redirect_to_name}.html"

        if type(app.builder) == DirectoryHTMLBuilder:
            if redirect_from_name != "index":
                redirect_from = (
                    src_redirect_from.parent / redirect_from_name / "index.html"
                )
            if redirect_to_name != "index":
                redirect_to = src_redirect_to.parent / redirect_to_name / "index.html"

        # absolute paths into the build dir
        build_redirect_from = Path(app.outdir) / redirect_from
        build_redirect_to = Path(app.outdir) / redirect_to

        if (
            build_redirect_from.exists()
            and src_redirect_from.as_posix() in redirect_record
        ):
            # if it is still pointing to the same source, continue
            if (
                redirect_record[src_redirect_from.as_posix()]
                == src_redirect_to.as_posix()
            ):
                continue
            # otherwise remove and rewrite
            build_redirect_from.unlink()

        if build_redirect_from.exists():
            logger.warning(
                f'{yellow("(broken)")} {redirect_from} redirects to {redirect_to} but {build_redirect_from} already exists!'
            )
            app.statuscode = 1
            continue

        if not build_redirect_to.exists():
            logger.warning(
                f'{yellow("(broken)")} {redirect_from} redirects to {redirect_to} but {build_redirect_to} does not exist!'
            )
            app.statuscode = 1
            continue

        build_redirect_from.parent.mkdir(parents=True, exist_ok=True)
        with build_redirect_from.open("w") as f:
            f.write(
                rediraffe_template.render(
                    rel_url=str(
                        PurePosixPath(
                            PureWindowsPath(
                                relpath(build_redirect_to, build_redirect_from.parent)
                            )
                        )
                    ),
                    from_file=src_redirect_from,
                    to_file=src_redirect_to,
                    from_url=redirect_from,
                    to_url=redirect_to,
                )
            )
            logger.info(
                f'{green("(good)")} {redirect_from} {green("-->")} {redirect_to}'
            )
            redirect_record[src_redirect_from.as_posix()] = src_redirect_to.as_posix()

    redirect_json_file.write_text(json.dumps(redirect_record), encoding="utf8")


class CheckRedirectsDiffBuilder(Builder):
    name = "rediraffecheckdiff"

    def init(self) -> None:
        super().init()

        source_suffixes = set(self.app.config.source_suffix)
        src_path = Path(self.app.srcdir)

        rediraffe_redirects = self.app.config.rediraffe_redirects
        redirects_path = None
        if isinstance(rediraffe_redirects, dict):
            pass
        elif isinstance(rediraffe_redirects, str):
            redirects_path = Path(src_path) / rediraffe_redirects
            if not redirects_path.is_file():
                logger.error(red("rediraffe: rediraffe_redirects file does not exist."))
                self.app.statuscode = 1
                return
            try:
                rediraffe_redirects = create_graph(redirects_path)
            except ExtensionError as e:
                self.app.statuscode = 1
                return
        else:
            logger.error("rediraffe: rediraffe was not given redirects to process.")
            self.app.statuscode = 1
            return

        absolute_redirects = {
            (src_path / redirect_from).resolve(): (src_path / redirect_to).resolve()
            for redirect_from, redirect_to in rediraffe_redirects.items()
        }

        path_to_git_repo = subprocess.check_output(
            f"git -C {self.app.srcdir} rev-parse --show-toplevel", shell=True
        ).decode("utf-8")

        def abs_path_in_src_dir_w_src_suffix(filename: str) -> Union[Path, None]:
            abs_path = (Path(path_to_git_repo.strip()) / filename.strip()).resolve()
            if not str(abs_path).startswith(str(src_path)):
                return None
            if abs_path.suffix not in source_suffixes:
                return None
            return abs_path

        # run git diff
        renamed_files_out = (
            subprocess.check_output(
                f"git -C {self.app.srcdir} diff --name-status --diff-filter=R {self.app.config.rediraffe_branch}",
                shell=True,
            )
            .decode("utf-8")
            .splitlines()
        )

        rename_hints = {}
        for line in renamed_files_out:
            line = line.strip()
            r_perc, rename_from, rename_to = re.split(r"\s+", line)
            perc = int(r_perc[1:])
            path_rename_from = abs_path_in_src_dir_w_src_suffix(rename_from)
            path_rename_to = abs_path_in_src_dir_w_src_suffix(rename_to)

            if path_rename_from == None:
                continue
            if path_rename_to == None:
                continue
            rename_hints[path_rename_from] = (path_rename_to, perc)

        # run git diff
        deleted_files = (
            subprocess.check_output(
                f"git -C {self.app.srcdir} diff --diff-filter=D --name-only {self.app.config.rediraffe_branch}",
                shell=True,
            )
            .decode("utf-8")
            .splitlines()
        )

        # to absolute path + filter out
        deleted_files = [
            abs_path_in_src_dir_w_src_suffix(filename) for filename in deleted_files
        ]
        deleted_files = list(filter(lambda x: x != None, deleted_files))

        for deleted_file in deleted_files:
            if deleted_file in absolute_redirects:
                logger.info(
                    f"deleted file {deleted_file} redirects to {absolute_redirects[deleted_file]}."
                )
            else:
                err_msg = f'{red("(broken)")} {deleted_file} was deleted but is not redirected!'
                logger.error(err_msg)
                self.app.statuscode = 1

        with redirects_path.open("a") as redirects_file:

            for renamed_file in rename_hints:
                hint_to, perc = rename_hints[renamed_file]

                if renamed_file in absolute_redirects:
                    logger.info(
                        f"renamed file {renamed_file} redirects to {absolute_redirects[renamed_file]}."
                    )
                    continue

                if self.name == "rediraffewritediff":
                    if perc >= self.app.config.rediraffe_auto_redirect_perc:
                        rel_rename_from = f'"{str(PurePosixPath(renamed_file.relative_to(src_path)))}"'
                        rel_rename_to = (
                            f'"{str(PurePosixPath(hint_to.relative_to(src_path)))}"'
                        )
                        redirects_file.write(f"{rel_rename_from} {rel_rename_to}\n")
                        logger.info(
                            f"{green('(okay)')} Renamed file {rel_rename_from} has been redirected to {rel_rename_to} in your redirects file!"
                        )
                        continue

                err_msg = (
                    f"{red('(broken)')} {renamed_file} was deleted but is not redirected!"
                    f" Hint: This file was renamed to {hint_to} with a similarity of {perc}%."
                )
                logger.error(err_msg)
                self.app.statuscode = 1

    def get_outdated_docs(self):
        return []

    def prepare_writing(self=None, docnames=None):
        pass

    def write_doc(self, docname=None, doctree=None):
        pass

    def get_target_uri(self, docname=None, typ=None):
        return ""

    def read(self):
        return []


class WriteRedirectsDiffBuilder(CheckRedirectsDiffBuilder):
    name = "rediraffewritediff"

    def init(self) -> None:
        rediraffe_redirects = self.app.config.rediraffe_redirects
        if not isinstance(rediraffe_redirects, str):
            logger.error(
                f"{red('(broken)')} Automatic redirects is only available with a redirects file."
            )
            self.app.statuscode = 1
            return

        super().init()


def setup(app: Sphinx) -> Dict[str, Any]:
    app.add_config_value("rediraffe_redirects", None, None)
    app.add_config_value("rediraffe_branch", "", None)
    app.add_config_value("rediraffe_template", None, None)
    app.add_config_value("rediraffe_auto_redirect_perc", 100, None)

    app.add_builder(CheckRedirectsDiffBuilder)
    app.add_builder(WriteRedirectsDiffBuilder)
    app.connect("build-finished", build_redirects)

    return {
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



from sphinxext.rediraffe import create_graph
import pytest
from sphinx.errors import ExtensionError


def test_create_graph(tmp_path):
    path = tmp_path / "rediraffe.txt"
    path.write_text(
        """
        a b
        c d
        d e
        """
    )
    graph = create_graph(path)
    assert graph == {
        "a": "b",
        "c": "d",
        "d": "e",
    }


def test_create_graph_spacing(tmp_path):
    path = tmp_path / "rediraffe.txt"
    path.write_text(
        """
        a  b
        c d
        d            e
        """
    )
    graph = create_graph(path)
    assert graph == {
        "a": "b",
        "c": "d",
        "d": "e",
    }


def test_create_graph_link_redirected_twice(tmp_path):
    path = tmp_path / "rediraffe.txt"
    path.write_text(
        """
        a b
        a c
        """
    )
    with pytest.raises(ExtensionError):
        graph = create_graph(path)


def test_create_graph_link_redirected_lots(tmp_path):
    path = tmp_path / "rediraffe.txt"
    path.write_text(
        """
        a b
        c d
        a c
        d e
        a q
        """
    )
    with pytest.raises(ExtensionError):
        graph = create_graph(path)


class TestCreateGraphQuotes:
    def test_no_quotes(self, tmp_path):
        path = tmp_path / "rediraffe.txt"
        path.write_text(
            """
            a b
            c d
            d e
            """
        )
        graph = create_graph(path)
        assert graph == {
            "a": "b",
            "c": "d",
            "d": "e",
        }

    def test_single_quotes(self, tmp_path):
        path = tmp_path / "rediraffe.txt"
        path.write_text(
            """
            'a' b
            c 'd'
            d e
            """
        )
        graph = create_graph(path)
        assert graph == {
            "a": "b",
            "c": "d",
            "d": "e",
        }

    def test_both_single_quotes(self, tmp_path):
        path = tmp_path / "rediraffe.txt"
        path.write_text(
            """
            'a' 'b'
            'c' 'd'
            d e
            """
        )
        graph = create_graph(path)
        assert graph == {
            "a": "b",
            "c": "d",
            "d": "e",
        }

    def test_commented(self, tmp_path):
        path = tmp_path / "rediraffe.txt"
        path.write_text(
            """
            # a comment
            a b
            c d
            # another comment
            d e
            """
        )
        graph = create_graph(path)
        assert graph == {
            "a": "b",
            "c": "d",
            "d": "e",
        }

    def test_double_quotes(self, tmp_path):
        path = tmp_path / "rediraffe.txt"
        path.write_text(
            """
            "a" b
            c "d"
            d e
            """
        )
        graph = create_graph(path)
        assert graph == {
            "a": "b",
            "c": "d",
            "d": "e",
        }

    def test_both_double_quotes(self, tmp_path):
        path = tmp_path / "rediraffe.txt"
        path.write_text(
            """
            "a" "b"
            "c" "d"
            d e
            """
        )
        graph = create_graph(path)
        assert graph == {
            "a": "b",
            "c": "d",
            "d": "e",
        }

    def test_quote_in_path(self, tmp_path):
        path = tmp_path / "rediraffe.txt"
        path.write_text(
            """
            "a b
            c d'
            d "e
            "e' f'"
            """
        )
        graph = create_graph(path)
        assert graph == {
            '"a': "b",
            "c": "d'",
            "d": '"e',
            "\"e'": "f'\"",
        }

    def test_complex(self, tmp_path):
        path = tmp_path / "rediraffe.txt"
        path.write_text(
            """
            "Double Quoted Path" 'Single Quoted Path'
            "Website's Contents" "other"
            'Store's Contents' other
            "quoteskept' "other"
            ""I'm ready! I'm ready!" - Spongebob Squarepants.rst" "just why?.rst"
            """
        )
        graph = create_graph(path)
        assert graph == {
            "Double Quoted Path": "Single Quoted Path",
            "Website's Contents": "other",
            "Store's Contents": "other",
            "\"quoteskept'": "other",
            "\"I'm ready! I'm ready!\" - Spongebob Squarepants.rst": "just why?.rst",
        }



from sphinxext.rediraffe import create_simple_redirects
import pytest
from sphinx.errors import ExtensionError


def test_create_simple_redirects_empty():
    assert create_simple_redirects({}) == {}


def test_create_simple_redirects_no_cycle():
    simple_redirects = redirects = {
        "a": "b",
    }
    assert create_simple_redirects(redirects) == simple_redirects


def test_create_simple_redirects_simple_cycle():
    redirects = {
        "a": "b",
        "b": "a",
    }

    with pytest.raises(ExtensionError):
        create_simple_redirects(redirects)


def test_create_simple_redirects_complex_cycles():
    redirects = {
        "a": "b",
        "b": "c",
        "c": "d",
        "d": "e",
        "e": "a",
    }

    with pytest.raises(ExtensionError):
        create_simple_redirects(redirects)


def test_create_simple_redirects_multiple_cycles():
    redirects = {
        "a": "b",
        "b": "c",
        "c": "d",
        "d": "e",
        "e": "a",
        "f": "g",
        "g": "h",
        "h": "j",
        "j": "g",
    }

    with pytest.raises(ExtensionError):
        create_simple_redirects(redirects)


def test_create_simple_redirects_no_chains():
    simple_redirects = redirects = {
        "a": "b",
        "c": "d",
        "e": "f",
    }
    assert create_simple_redirects(redirects) == simple_redirects


def test_create_simple_redirects_chain():
    redirects = {
        "a": "b",
        "b": "c",
        "c": "d",
    }

    simple_redirects = {
        "a": "d",
        "b": "d",
        "c": "d",
    }
    assert create_simple_redirects(redirects) == simple_redirects


def test_create_simple_redirects_mixed_chains():
    redirects = {
        "a": "b",
        "b": "c",
        "c": "d",
        "e": "f",
        "g": "h",
        "h": "i",
    }

    simple_redirects = {
        "a": "d",
        "b": "d",
        "c": "d",
        "e": "f",
        "g": "i",
        "h": "i",
    }
    assert create_simple_redirects(redirects) == simple_redirects



import subprocess
import setuptools

try:
    ret = subprocess.check_output(
        "git describe --tags --abbrev=0",
        shell=True,
    )
    version = ret.decode("utf-8").strip()
except:
    version = "main"

with open("README.md", "r", encoding="utf-8") as readme:
    long_description = readme.read()

setuptools.setup(
    name="sphinxext-rediraffe",
    version=version,
    author="Vasista Vovveti",
    author_email="vasistavovveti@gmail.com",
    description="Sphinx Extension that redirects non-existent pages to working pages.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/wpilibsuite/sphinxext-rediraffe",
    install_requires=["sphinx>=2.0"],
    packages=["sphinxext"],
    classifiers=[
        "Environment :: Plugins",
        "Environment :: Web Environment",
        "Framework :: Sphinx :: Extension",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python",
        "Topic :: Documentation :: Sphinx",
        "Topic :: Documentation",
        "Topic :: Software Development :: Documentation",
        "Topic :: Text Processing",
        "Topic :: Utilities",
    ],
    python_requires=">=3.6",
    include_package_data=True,
)



extensions = ["sphinxext.rediraffe"]

master_doc = "index"
exclude_patterns = ["_build"]

html_theme = "basic"

rediraffe_redirects = {
    "f.rst": "c.rst",
    "q.rst": "w.rst",
    "F2/1.rst": "x.rst",
    "c.rst": "d.rst",
    "i.rst": "j.rst",
    "k.rst": "l.rst",
    "r.rst": "w.rst",
    "m.rst": "n.rst",
    "n.rst": "o.rst",
    "u.rst": "w.rst",
    "F1/1.rst": "F1/2.rst",
    "t.rst": "w.rst",
    "b.rst": "c.rst",
    "g.rst": "d.rst",
    "h.rst": "g.rst",
    "p.rst": "z.rst",
    "a.rst": "b.rst",
    "d.rst": "e.rst",
    "w.rst": "x.rst",
    "y.rst": "x.rst",
    "x.rst": "z.rst",
    "v.rst": "u.rst",
    "F1/2.rst": "F2/1.rst",
    "s.rst": "w.rst",
    "F5/F4/F3/F2/F1/1.rst": "index.rst",
}



import pytest
import seleniumbase
from seleniumbase import BaseCase
from sphinx.testing.path import path
from sphinx.application import Sphinx
from sphinx.errors import ExtensionError
from pathlib import Path
import shutil
import logging

from conftest import rel2url


@pytest.fixture(scope="module")
def rootdir():
    return path(__file__).parent.abspath() / "roots/ext"


class TestExtHtml:
    @pytest.mark.sphinx("html", testroot="no_redirects")
    def test_no_redirects(self, app: Sphinx):
        app.build()
        assert app.statuscode == 0

    @pytest.mark.sphinx("html", testroot="simple")
    def test_simple(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("another.html", "index.html")

    @pytest.mark.sphinx("html", testroot="simple")
    def test_simple_rebuild(self, app_params, make_app, ensure_redirect):
        args, kwargs = app_params
        app = make_app(*args, **kwargs)
        if Path(app.outdir).exists():
            shutil.rmtree(Path(app.outdir))
        app.build()
        assert app.statuscode == 0
        app2 = make_app(*args, **kwargs)
        app2.build()
        assert app2.statuscode == 0
        ensure_redirect("another.html", "index.html")

    @pytest.mark.sphinx("html", testroot="no_cycle")
    def test_no_cycle(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("a.html", "index.html")
        ensure_redirect("b.html", "index.html")

    @pytest.mark.sphinx("html", testroot="cycle")
    def test_cycle(self, app: Sphinx):
        with pytest.raises(ExtensionError):
            app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("html", testroot="nested")
    def test_nested(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("tof1.html", "docs/folder1/f1.html")
        ensure_redirect("docs/folder1/tof1.html", "docs/folder1/f1.html")
        ensure_redirect("docs/folder1/tof2.html", "docs/folder2/f2.html")
        ensure_redirect("docs/folder2/toindex.html", "index.html")
        ensure_redirect("totoindex.html", "index.html")

    @pytest.mark.sphinx("html", testroot="backslashes")
    def test_backslashes(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("tof1.html", "docs/folder1/f1.html")
        ensure_redirect("docs/folder1/tof1.html", "docs/folder1/f1.html")
        ensure_redirect("docs/folder1/tof2.html", "docs/folder2/f2.html")
        ensure_redirect("docs/folder2/toindex.html", "index.html")
        ensure_redirect("totoindex.html", "index.html")

    @pytest.mark.sphinx("html", testroot="dot_in_filename")
    def test_dot_in_filename(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("docs/x.y.z.html", "docs/a.b.c.html")

    @pytest.mark.sphinx("html", testroot="mixed_slashes")
    def test_mixed_slashes(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("tof1.html", "docs/folder1/f1.html")
        ensure_redirect("docs/folder1/tof1.html", "docs/folder1/f1.html")
        ensure_redirect("docs/folder1/tof2.html", "docs/folder2/f2.html")
        ensure_redirect("docs/folder2/toindex.html", "index.html")
        ensure_redirect("totoindex.html", "index.html")

    @pytest.mark.sphinx("html", testroot="link_redirected_twice")
    def test_link_redirected_twice(self, app: Sphinx):
        with pytest.raises(ExtensionError):
            app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("html", testroot="link_redirected_to_nonexistant_file")
    def test_link_redirected_to_nonexistant_file(self, app: Sphinx):
        app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("html", testroot="existing_link_redirected")
    def test_existing_link_redirected(self, app: Sphinx):
        app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("html", testroot="bad_rediraffe_file")
    def test_bad_rediraffe_file(self, app: Sphinx):
        app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("html", testroot="no_rediraffe_file")
    def test_no_rediraffe_file(self, app: Sphinx):
        app.build()
        assert app.statuscode == 0
        assert "rediraffe was not given redirects to process" in app._warning.getvalue()

    @pytest.mark.sphinx("html", testroot="redirect_from_deleted_folder")
    def test_redirect_from_deleted_folder(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0

        ensure_redirect("deletedfolder/another.html", "index.html")
        ensure_redirect("deletedfolder/deletedfolder2/another.html", "index.html")

    @pytest.mark.sphinx("html", testroot="complex")
    def test_complex(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0

        ensure_redirect("a.html", "e.html")
        ensure_redirect("b.html", "e.html")
        ensure_redirect("c.html", "e.html")
        ensure_redirect("d.html", "e.html")
        ensure_redirect("f.html", "e.html")
        ensure_redirect("g.html", "e.html")
        ensure_redirect("h.html", "e.html")

        ensure_redirect("i.html", "j.html")

        ensure_redirect("k.html", "l.html")

        ensure_redirect("m.html", "o.html")
        ensure_redirect("n.html", "o.html")

        ensure_redirect("q.html", "z.html")
        ensure_redirect("r.html", "z.html")
        ensure_redirect("s.html", "z.html")
        ensure_redirect("t.html", "z.html")
        ensure_redirect("u.html", "z.html")
        ensure_redirect("v.html", "z.html")
        ensure_redirect("w.html", "z.html")
        ensure_redirect("x.html", "z.html")
        ensure_redirect("y.html", "z.html")
        ensure_redirect("F1/1.html", "z.html")
        ensure_redirect("F1/2.html", "z.html")
        ensure_redirect("F2/1.html", "z.html")

        ensure_redirect("F5/F4/F3/F2/F1/1.html", "index.html")

    @pytest.mark.sphinx("html", testroot="complex_dict")
    def test_complex_dict(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0

        ensure_redirect("a.html", "e.html")
        ensure_redirect("b.html", "e.html")
        ensure_redirect("c.html", "e.html")
        ensure_redirect("d.html", "e.html")
        ensure_redirect("f.html", "e.html")
        ensure_redirect("g.html", "e.html")
        ensure_redirect("h.html", "e.html")

        ensure_redirect("i.html", "j.html")

        ensure_redirect("k.html", "l.html")

        ensure_redirect("m.html", "o.html")
        ensure_redirect("n.html", "o.html")

        ensure_redirect("q.html", "z.html")
        ensure_redirect("r.html", "z.html")
        ensure_redirect("s.html", "z.html")
        ensure_redirect("t.html", "z.html")
        ensure_redirect("u.html", "z.html")
        ensure_redirect("v.html", "z.html")
        ensure_redirect("w.html", "z.html")
        ensure_redirect("x.html", "z.html")
        ensure_redirect("y.html", "z.html")
        ensure_redirect("F1/1.html", "z.html")
        ensure_redirect("F1/2.html", "z.html")
        ensure_redirect("F2/1.html", "z.html")

        ensure_redirect("F5/F4/F3/F2/F1/1.html", "index.html")

    @pytest.mark.sphinx("html", testroot="jinja")
    def test_jinja(self, app: Sphinx, _sb: BaseCase):
        app.build()
        assert app.statuscode == 0

        _sb.open(rel2url(app.outdir, "another.html"))
        text = _sb.get_text(selector="html")
        text = text.replace("\\", "/")
        text = text.replace("//", "/")

        assert "rel_url: index.html" in text
        assert "from_file: another.rst" in text
        assert "to_file: index.rst" in text
        assert "from_url: another.html" in text
        assert "to_url: index.html" in text

    @pytest.mark.sphinx("html", testroot="jinja_bad_path")
    def test_jinja_bad_path(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0

        ensure_redirect("another.html", "index.html")

    @pytest.mark.sphinx("html", testroot="pass_url_fragments_queries")
    def test_pass_url_fragments(self, app: Sphinx, _sb: BaseCase, ensure_redirect):
        app.build()

        ensure_redirect("another.html", "index.html")
        _sb.open(rel2url(app.outdir, "another.html") + "#haha")
        # check url
        assert Path(rel2url(app.outdir, "index.html")) == Path(
            _sb.execute_script(
                'return  window.location.protocol + "//" + window.location.host + "/" + window.location.pathname'
            )
        )
        # check hash
        assert "#haha" == _sb.execute_script("return window.location.hash")

    @pytest.mark.sphinx("html", testroot="pass_url_fragments_queries")
    def test_pass_url_queries(self, app: Sphinx, _sb: BaseCase, ensure_redirect):
        app.build()

        ensure_redirect("another.html", "index.html")
        _sb.open(rel2url(app.outdir, "another.html") + "?phrase=haha")
        # check url
        assert Path(rel2url(app.outdir, "index.html")) == Path(
            _sb.execute_script(
                'return window.location.protocol + "//" + window.location.host + "/" + window.location.pathname'
            )
        )
        # check query
        assert "?phrase=haha" == _sb.execute_script("return window.location.search")

    @pytest.mark.sphinx("html", testroot="pass_url_fragments_queries")
    def test_pass_url_fragment_and_query(
        self, app: Sphinx, _sb: BaseCase, ensure_redirect
    ):
        app.build()

        ensure_redirect("another.html", "index.html")
        _sb.open(rel2url(app.outdir, "another.html") + "?phrase=haha#giraffe")
        # check url
        assert Path(rel2url(app.outdir, "index.html")) == Path(
            _sb.execute_script(
                'return window.location.protocol + "//" + window.location.host + "/" + window.location.pathname'
            )
        )
        # check query
        assert "?phrase=haha" == _sb.execute_script("return window.location.search")
        # check hash
        assert "#giraffe" == _sb.execute_script("return window.location.hash")


class TestExtDirHtml:
    @pytest.mark.sphinx("dirhtml", testroot="no_redirects")
    def test_no_redirects(self, app: Sphinx):
        app.build()
        assert app.statuscode == 0

    @pytest.mark.sphinx("dirhtml", testroot="simple")
    def test_simple(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("another/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="dirhtml_user_index_files")
    def test_index_file_foldering(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("another/index.html", "mydir/index.html")

    @pytest.mark.sphinx("dirhtml", testroot="simple", freshenv=False)
    def test_simple_rebuild(self, app_params, make_app, ensure_redirect):
        args, kwargs = app_params
        app = make_app(*args, **kwargs)
        if Path(app.outdir).exists():
            shutil.rmtree(Path(app.outdir))
        app.build()
        assert app.statuscode == 0
        app2 = make_app(*args, **kwargs)
        app2.build()
        assert app2.statuscode == 0
        ensure_redirect("another/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="no_cycle")
    def test_no_cycle(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("a/index.html", "index.html")
        ensure_redirect("b/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="cycle")
    def test_cycle(self, app: Sphinx):
        with pytest.raises(ExtensionError):
            app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("dirhtml", testroot="nested")
    def test_nested(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("tof1/index.html", "docs/folder1/f1/index.html")
        ensure_redirect("docs/folder1/tof1/index.html", "docs/folder1/f1/index.html")
        ensure_redirect("docs/folder1/tof2/index.html", "docs/folder2/f2/index.html")
        ensure_redirect("docs/folder2/toindex/index.html", "index.html")
        ensure_redirect("totoindex/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="backslashes")
    def test_backslashes(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("tof1/index.html", "docs/folder1/f1/index.html")
        ensure_redirect("docs/folder1/tof1/index.html", "docs/folder1/f1/index.html")
        ensure_redirect("docs/folder1/tof2/index.html", "docs/folder2/f2/index.html")
        ensure_redirect("docs/folder2/toindex/index.html", "index.html")
        ensure_redirect("totoindex/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="dot_in_filename")
    def test_dot_in_filename(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("docs/x.y.z/index.html", "docs/a.b.c/index.html")

    @pytest.mark.sphinx("dirhtml", testroot="mixed_slashes")
    def test_mixed_slashes(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0
        ensure_redirect("tof1/index.html", "docs/folder1/f1/index.html")
        ensure_redirect("docs/folder1/tof1/index.html", "docs/folder1/f1/index.html")
        ensure_redirect("docs/folder1/tof2/index.html", "docs/folder2/f2/index.html")
        ensure_redirect("docs/folder2/toindex/index.html", "index.html")
        ensure_redirect("totoindex/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="link_redirected_twice")
    def test_link_redirected_twice(self, app: Sphinx):
        with pytest.raises(ExtensionError):
            app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("dirhtml", testroot="link_redirected_to_nonexistant_file")
    def test_link_redirected_to_nonexistant_file(self, app: Sphinx):
        app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("dirhtml", testroot="existing_link_redirected")
    def test_existing_link_redirected(self, app: Sphinx):
        app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("dirhtml", testroot="bad_rediraffe_file")
    def test_bad_rediraffe_file(self, app: Sphinx):
        app.build()
        assert app.statuscode == 1

    @pytest.mark.sphinx("dirhtml", testroot="no_rediraffe_file")
    def test_no_rediraffe_file(self, app: Sphinx):
        app.build()
        assert app.statuscode == 0
        assert "rediraffe was not given redirects to process" in app._warning.getvalue()

    @pytest.mark.sphinx("dirhtml", testroot="redirect_from_deleted_folder")
    def test_redirect_from_deleted_folder(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0

        ensure_redirect("deletedfolder/another/index.html", "index.html")
        ensure_redirect("deletedfolder/deletedfolder2/another/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="complex")
    def test_complex(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0

        ensure_redirect("a/index.html", "e/index.html")
        ensure_redirect("b/index.html", "e/index.html")
        ensure_redirect("c/index.html", "e/index.html")
        ensure_redirect("d/index.html", "e/index.html")
        ensure_redirect("f/index.html", "e/index.html")
        ensure_redirect("g/index.html", "e/index.html")
        ensure_redirect("h/index.html", "e/index.html")

        ensure_redirect("i/index.html", "j/index.html")

        ensure_redirect("k/index.html", "l/index.html")

        ensure_redirect("m/index.html", "o/index.html")
        ensure_redirect("n/index.html", "o/index.html")

        ensure_redirect("q/index.html", "z/index.html")
        ensure_redirect("r/index.html", "z/index.html")
        ensure_redirect("s/index.html", "z/index.html")
        ensure_redirect("t/index.html", "z/index.html")
        ensure_redirect("u/index.html", "z/index.html")
        ensure_redirect("v/index.html", "z/index.html")
        ensure_redirect("w/index.html", "z/index.html")
        ensure_redirect("x/index.html", "z/index.html")
        ensure_redirect("y/index.html", "z/index.html")
        ensure_redirect("F1/1/index.html", "z/index.html")
        ensure_redirect("F1/2/index.html", "z/index.html")
        ensure_redirect("F2/1/index.html", "z/index.html")

        ensure_redirect("F5/F4/F3/F2/F1/1/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="complex_dict")
    def test_complex_dict(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0

        ensure_redirect("a/index.html", "e/index.html")
        ensure_redirect("b/index.html", "e/index.html")
        ensure_redirect("c/index.html", "e/index.html")
        ensure_redirect("d/index.html", "e/index.html")
        ensure_redirect("f/index.html", "e/index.html")
        ensure_redirect("g/index.html", "e/index.html")
        ensure_redirect("h/index.html", "e/index.html")

        ensure_redirect("i/index.html", "j/index.html")

        ensure_redirect("k/index.html", "l/index.html")

        ensure_redirect("m/index.html", "o/index.html")
        ensure_redirect("n/index.html", "o/index.html")

        ensure_redirect("q/index.html", "z/index.html")
        ensure_redirect("r/index.html", "z/index.html")
        ensure_redirect("s/index.html", "z/index.html")
        ensure_redirect("t/index.html", "z/index.html")
        ensure_redirect("u/index.html", "z/index.html")
        ensure_redirect("v/index.html", "z/index.html")
        ensure_redirect("w/index.html", "z/index.html")
        ensure_redirect("x/index.html", "z/index.html")
        ensure_redirect("y/index.html", "z/index.html")
        ensure_redirect("F1/1/index.html", "z/index.html")
        ensure_redirect("F1/2/index.html", "z/index.html")
        ensure_redirect("F2/1/index.html", "z/index.html")

        ensure_redirect("F5/F4/F3/F2/F1/1/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="jinja")
    def test_jinja(self, app: Sphinx, _sb: BaseCase):
        app.build()
        assert app.statuscode == 0

        _sb.open(rel2url(app.outdir, "another/index.html"))
        text = _sb.get_text(selector="html")
        text = text.replace("\\", "/")
        text = text.replace("//", "/")

        assert "rel_url: ../index.html" in text
        assert "from_file: another.rst" in text
        assert "to_file: index.rst" in text
        assert "from_url: another/index.html" in text
        assert "to_url: index.html" in text

    @pytest.mark.sphinx("dirhtml", testroot="jinja_bad_path")
    def test_jinja_bad_path(self, app: Sphinx, ensure_redirect):
        app.build()
        assert app.statuscode == 0

        ensure_redirect("another/index.html", "index.html")

    @pytest.mark.sphinx("dirhtml", testroot="pass_url_fragments_queries")
    def test_pass_url_fragments(self, app: Sphinx, _sb: BaseCase, ensure_redirect):
        app.build()

        ensure_redirect("another/index.html", "index.html")
        _sb.open(rel2url(app.outdir, "another/index.html") + "#haha")
        # check url
        assert Path(rel2url(app.outdir, "index.html")) == Path(
            _sb.execute_script(
                'return  window.location.protocol + "//" + window.location.host + "/" + window.location.pathname'
            )
        )
        # check hash
        assert "#haha" == _sb.execute_script("return window.location.hash")

    @pytest.mark.sphinx("dirhtml", testroot="pass_url_fragments_queries")
    def test_pass_url_queries(self, app: Sphinx, _sb: BaseCase, ensure_redirect):
        app.build()

        ensure_redirect("another/index.html", "index.html")
        _sb.open(rel2url(app.outdir, "another/index.html") + "?phrase=haha")
        # check url
        assert Path(rel2url(app.outdir, "index.html")) == Path(
            _sb.execute_script(
                'return window.location.protocol + "//" + window.location.host + "/" + window.location.pathname'
            )
        )
        # check query
        assert "?phrase=haha" == _sb.execute_script("return window.location.search")

    @pytest.mark.sphinx("dirhtml", testroot="pass_url_fragments_queries")
    def test_pass_url_fragment_and_query(
        self, app: Sphinx, _sb: BaseCase, ensure_redirect
    ):
        app.build()

        ensure_redirect("another/index.html", "index.html")
        _sb.open(rel2url(app.outdir, "another/index.html") + "?phrase=haha#giraffe")
        # check url
        assert Path(rel2url(app.outdir, "index.html")) == Path(
            _sb.execute_script(
                'return window.location.protocol + "//" + window.location.host + "/" + window.location.pathname'
            )
        )
        # check query
        assert "?phrase=haha" == _sb.execute_script("return window.location.search")
        # check hash
        assert "#giraffe" == _sb.execute_script("return window.location.hash")



extensions = ["sphinxext.rediraffe"]

master_doc = "index"
exclude_patterns = ["_build"]

html_theme = "basic"

rediraffe_branch = "HEAD~1"
rediraffe_redirects = "redirects.txt"
rediraffe_auto_redirect_perc = 50
